#!/usr/bin/env python
#
# Data Quality Checker also known as DQC
#
# Read the README.md for more detailed information
#
#       Barcelona Supercomputing Center - Centro Nacional de Supercomputacion
#       Earth Sciences Department - Computational Earth Sciences
#     
#     Authors: 
#       Joan Sala Calero (joan.sala@bsc.es)
#       Jesus Pena-Izquierdo (jesus.pena@bsc.es)

"""
Script: Generate a list of QAR ini configuration files
---
- Read from download lists
- Generate ini file to be used by the
"""

import os
import json
import glob
import requests

def generate_ini(rootd, pattern, variable_e, datatype, nbp, variable, res, ts, origin, system, dataset):
	input_dir = ''.format(ds=dataset, v=variable)

	ini='''[general]
input = /shared/cds_downloads/{r}/{ds}/{v}
fpattern = {p}
log_dir = /shared/QAR_dqc/logs 
res_dir = /shared/QAR_dqc/results
forms_dir = /shared/cds-forms-c3s

[dataset]
variable = {ve}
datatype = {dt}
numberofpoints = {np}
cds_dataset = {ds}
cds_variable = {v}

[file_format]

[standard_compliance]

[spatial_completeness]
mask_file =
mask_var =
mask_dim =

[temporal_completeness]
origin = {o}
system = {s}
is_seasonal = monthly

[spatial_consistency]
grid_interval = {res}
grid_type = lonlat

[temporal_consistency]
time_step = {t}
time_granularity = month

[valid_ranges]
valid_min =
valid_max =	
'''.format(
	p=pattern, ve=variable_e, dt=datatype, v=variable, np=nbp,
	o=origin, s=system, ds=dataset, t=ts, r=rootd, res=res)

	return ini


"""
Given a cds variable name (long-name)
Return a parameter identifier in the mapping.json
"""
def get_qar_names(jsonf, codes_var):
	qar_names = []
	inis = []
	with open(jsonf, 'r') as f:
		dd=json.load(f)
		
		for dl in dd:
			for v in dl['variables']:				

				d = dl['dataset']
				ve = codes_var[dl['dataset']]['variables'][v]	

				# Seasonal
				if 'originating_centre' in dl['form']:
					root = 'seasonal'
					res = 1
					dt = 'em, fcmean, hcmean'
					o = dl['form']['originating_centre']
					s = dl['form']['system']					
					patt = o+'-'+s+'*.grib'
					
					qar_name = d+'-'+v+'-'+o+'-'+s+'.ini'
					ini = generate_ini(root, patt, ve, dt, '', v, res, 1, o, s, d)
						
				# Reanalysis
				else:	
					root = 'reanalysis'	
					res = 0.25
					patt = 'monthly_averaged_reanalysis-*.grib'
					dt = 'an'
					qar_name = d+'-'+v+'.ini'	

					# Weird shit
					if 'levels-monthly-means' in d:
						nbp = 1038240
					else:
						nbp = ''

					ini = generate_ini(root, patt, ve, dt, nbp, v, res, 1, '', '', d)

				# Next
				qar_names.append(qar_name)
				inis.append(ini)

	return zip(qar_names, inis)

def write_file(outf, string):
	with open(outf, 'w') as f:
		f.write(string)

def main():

	# Config paths
	conf_dir="/mnt/c/BSC/c3s512-wp1-toolbox/conf"
	qar_dir="/mnt/c/BSC/c3s512-wp1-datachecker/dqc_wrapper/QAR_dqc/conf"
	out_dir="/mnt/c/Users/Joan/Desktop/QAR_results/conf_generate"
	datasets="/mnt/c/BSC/c3s512-wp1-datachecker/dqc_wrapper/grib/datasets.json"

	# Get codes
	codes_var = json.load(open(datasets,'r'))

	# For downloaded datasets
	for json_conf in glob.glob(conf_dir+'/*monthly*json'):

		# Get file names
		qar_names = get_qar_names(json_conf, codes_var)		
		for data in qar_names:
			q,i = data
			qar=os.path.join(qar_dir, q)
			qar_out=os.path.join(out_dir, q)
			
			if os.path.exists(qar_out):
				print('Skipping ' + q)				
			else:
				print('Writting ' + q)
				write_file(qar_out, i)

			if os.path.exists(qar):
				os.system('diff -b {} {}'.format(qar_out, qar))

if __name__ == "__main__":
    main()